@@ -14,7 +14,6 @@
 
 void makeTable_dialog_viewset(int pCnt);
 void makeList_dialog_viewset(int pCnt);
-GtkWidget *create_expandedwindow(void);
 void expanding_screen(void);
 GtkWidget *create_rightclickmenu1 (gpointer user_data);
 GtkWidget *create_rightclickmenu2 (gpointer user_data);
@@ -46,8 +45,7 @@ extern IMAGEINFO *iinfo_ex;
 
 GtkWidget *window_ex;
 extern int exwindowWidth, exwindowHeight;
-GtkWidget *label_ex;
-GtkWidget *window_ex_togglebutton;
+GtkWidget *drawingarea_ex;
 
 GtkWidget *dialog_send;
 int maxcastnum = 0;
@@ -62,9 +60,6 @@ GtkWidget *dialog_send_table2;
 GtkWidget *dialog_send_label_ID;
 GtkWidget *dialog_send_comboboxentry1;
 GtkWidget *dialog_send_comboboxentry2;
-extern GtkWidget *dialog_send_viewport1;
-extern GtkWidget *dialog_send_viewport2;
-extern GtkWidget *dialog_send_hbox;
 char fullscreen_flag;
 
 extern GtkWidget *dialog_IDinput;
@@ -72,12 +67,13 @@ char server_name[16];
 
 GtkWidget *dialog_lock;
 GtkWidget *dialog_lock_table;
-extern GtkWidget *dialog_lock_viewport;
 
 GtkWidget *dialog_quit;
 
 GtkWidget *dialog_version;
 
+GtkWidget *dialog_notice;
+
 /*******************************************************************
  *ᥤ󥦥ɥФ륤٥ȤΥХåؿ
  *******************************************************************/
@@ -203,6 +199,7 @@ on_main_drawingarea_button_press_event     (GtkWidget       *widget,
     if (event->type == GDK_2BUTTON_PRESS) {
       SETCTRL(clientNum, FALSE);
       expanding_screen();
+      GtkWidget *window_ex_togglebutton = lookup_widget(GTK_WIDGET(window_ex), "window_ex_togglebutton");
       gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(window_ex_togglebutton), FALSE);
     }
   }
@@ -270,6 +267,7 @@ on_viewset_activate                    (GtkMenuItem     *menuitem,
       dialog_viewset_clientnum[pCnt][dCnt] = EMPTY;
 
   dialog_viewset = create_dialog_viewset();
+  GtkWidget *dialog_viewset_viewport = lookup_widget(GTK_WIDGET(dialog_viewset), "dialog_viewset_viewport");
   dialog_viewset_table = gtk_table_new (2*tab_num-1, tab_num, FALSE);
   gtk_widget_show (dialog_viewset_table);
   gtk_container_add (GTK_CONTAINER (dialog_viewset_viewport), dialog_viewset_table);
@@ -319,6 +317,9 @@ on_send_activate                       (GtkMenuItem     *menuitem,
 		      G_CALLBACK (on_dialog_send_checkbutton1_toggled),
 		      (gpointer) pCnt);
   }
+
+  GtkWidget *dialog_send_viewport1 = lookup_widget(GTK_WIDGET(dialog_send), "dialog_send_viewport1");
+  GtkWidget *dialog_send_viewport2 = lookup_widget(GTK_WIDGET(dialog_send), "dialog_send_viewport2");
   gtk_container_add (GTK_CONTAINER (dialog_send_viewport1), dialog_send_table1);
   gtk_widget_show (dialog_send_table1);
 
@@ -357,6 +358,7 @@ on_lock_activate                       (GtkMenuItem     *menuitem,
 		      G_CALLBACK (on_dialog_lock_checkbutton1_toggled),
 		      (gpointer) pCnt);
   }
+  GtkWidget *dialog_lock_viewport = lookup_widget(GTK_WIDGET(dialog_lock), "dialog_lock_viewport");
   gtk_container_add (GTK_CONTAINER (dialog_lock_viewport), dialog_lock_table);
   gtk_widget_show (dialog_lock_table);
 
@@ -365,14 +367,6 @@ on_lock_activate                       (GtkMenuItem     *menuitem,
 }
 
 
-void
-on_usage_activate                      (GtkMenuItem     *menuitem,
-                                        gpointer         user_data)
-{
-
-}
-
-
 void
 on_version_activate                    (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
@@ -562,6 +556,7 @@ on_dialog_viewset_delete_event                (GtkWidget       *widget,
 					       GdkEvent        *event,
 					       gpointer         user_data)
 {
+  GtkWidget *dialog_viewset = lookup_widget(GTK_WIDGET(dialog_viewset), "dialog_viewset");
   gtk_grab_remove(dialog_viewset);
   gtk_widget_destroy(dialog_viewset);
   dialog_viewset = NULL;
@@ -789,11 +784,36 @@ void
 expanding_screen()
 {
   if (window_ex == NULL) {
-    window_ex = create_expandedwindow();
+    SETVIEW(clientNum, TRUE);
+    clientNum_ex = clientNum;
+    window_ex = create_window_ex();
+    GtkWidget *label_ex = lookup_widget(GTK_WIDGET(window_ex), "label_ex");
+    GtkWidget *aspectframe = lookup_widget(GTK_WIDGET(window_ex), "aspectframe");
+    drawingarea_ex = lookup_widget(GTK_WIDGET(window_ex), "drawingarea_ex");
+
+    gtk_label_set_text (GTK_LABEL(label_ex), server_CB.client_grp[clientNum]->client_ID);
+    gtk_widget_show(label_ex);
+
+    float aspect = (float)server_CB.client_grp[clientNum_ex]->serverInitMsg.framebufferWidth / (float)server_CB.client_grp[clientNum_ex]->serverInitMsg.framebufferHeight;
+    gtk_aspect_frame_set(GTK_ASPECT_FRAME(aspectframe), 0.5, 0.5, aspect, FALSE);
+    gtk_widget_set_size_request (aspectframe, exwindowWidth, exwindowHeight);
+    gtk_widget_show(aspectframe);
+
+    gtk_widget_set_events (drawingarea_ex, GDK_EXPOSURE_MASK | GDK_POINTER_MOTION_MASK
+			| GDK_POINTER_MOTION_HINT_MASK | GDK_BUTTON_MOTION_MASK | GDK_BUTTON_PRESS_MASK
+			| GDK_BUTTON_RELEASE_MASK | GDK_ENTER_NOTIFY_MASK | GDK_LEAVE_NOTIFY_MASK
+			| GDK_FOCUS_CHANGE_MASK | GDK_STRUCTURE_MASK);
+
+    smallimage_ex = gdk_image_new (GDK_IMAGE_FASTEST, gdkvis,
+                         server_CB.client_grp[clientNum]->serverInitMsg.framebufferWidth,
+                         server_CB.client_grp[clientNum]->serverInitMsg.framebufferHeight);
+
     gtk_widget_show(window_ex);
   } else if (clientNum_ex != clientNum) {
     SETVIEW(clientNum_ex, FALSE);
     SETCTRL(clientNum_ex, FALSE);
+    GtkWidget *label_ex = lookup_widget(GTK_WIDGET(window_ex), "label_ex");
+    GtkWidget *window_ex_togglebutton = lookup_widget(GTK_WIDGET(window_ex), "window_ex_togglebutton");
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(window_ex_togglebutton), FALSE);
     clientNum_ex = clientNum;
     CreateImageInfo(TRUE);
@@ -804,94 +824,12 @@ expanding_screen()
 				   server_CB.client_grp[clientNum_ex]->serverInitMsg.framebufferHeight);
     gtk_label_set_text(GTK_LABEL(label_ex), server_CB.client_grp[clientNum_ex]->client_ID);
     clientNum = clientNum_ex;
+    draw_image(pixmap_ex, gdkGC, gdkimage[clientNum_ex], 0, 0, 0, 0,
+               gdkimage[clientNum_ex]->width, gdkimage[clientNum_ex]->height);
   }
 }
 
 
-/*
-  ɽѥɥκ
- */
-GtkWidget *create_expandedwindow(void)
-{
-  GtkWidget *window;
-  GtkWidget *vbox;
-  GtkWidget *hbox;
-
-  SETVIEW(clientNum, TRUE);
-  clientNum_ex = clientNum;
-
-  /* ɥκ */
-  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-  gtk_window_set_default_size (GTK_WINDOW(window), exwindowWidth, exwindowHeight);
-  gtk_window_set_title (GTK_WINDOW (window), _("Expanded screen"));
-
-  vbox = gtk_vbox_new (FALSE, 0);
-  gtk_widget_show (vbox);
-  gtk_container_add (GTK_CONTAINER (window), vbox);
-
-  hbox = gtk_hbox_new (FALSE, 0);
-  gtk_widget_show (hbox);
-  gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
-
-  /* 饤ȤIDɽlabelκ */
-  label_ex = gtk_label_new (_(server_CB.client_grp[clientNum]->client_ID));
-  gtk_widget_show (label_ex);
-  gtk_box_pack_start (GTK_BOX (hbox), label_ex, TRUE, TRUE, 0);
-
-  /* toggle button */
-  window_ex_togglebutton = gtk_toggle_button_new_with_mnemonic (_("Operation ON/OFF"));
-  gtk_widget_show (window_ex_togglebutton);
-  gtk_box_pack_start (GTK_BOX (hbox), window_ex_togglebutton, FALSE, FALSE, 0);
-  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(window_ex_togglebutton), FALSE);
-
-  /* drawing areaκ */
-  drawingarea_ex = gtk_drawing_area_new ();
-  gtk_widget_show (drawingarea_ex);
-  gtk_box_pack_start (GTK_BOX (vbox), drawingarea_ex, TRUE, TRUE, 0);
-  gtk_widget_set_events (drawingarea_ex, GDK_EXPOSURE_MASK | GDK_POINTER_MOTION_MASK
-			 | GDK_POINTER_MOTION_HINT_MASK | GDK_BUTTON_MOTION_MASK | GDK_BUTTON_PRESS_MASK
-			 | GDK_BUTTON_RELEASE_MASK | GDK_ENTER_NOTIFY_MASK | GDK_LEAVE_NOTIFY_MASK
-			 | GDK_FOCUS_CHANGE_MASK | GDK_STRUCTURE_MASK);
-
-  g_signal_connect ((gpointer) window, "delete_event",
-                    G_CALLBACK (on_window_ex_delete_event),
-                    NULL);
-  g_signal_connect ((gpointer) window, "key_press_event",
-                    G_CALLBACK (on_window_ex_key_press_event),
-                    NULL);
-  g_signal_connect ((gpointer) window, "key_release_event",
-                    G_CALLBACK (on_window_ex_key_release_event),
-                    NULL);
-  g_signal_connect ((gpointer) window, "focus_out_event",
-                    G_CALLBACK (on_window_ex_focus_out_event),
-                    NULL);
-  g_signal_connect ((gpointer) drawingarea_ex, "button_press_event",
-                    G_CALLBACK (on_drawingarea_ex_button_press_event),
-                    NULL);
-  g_signal_connect ((gpointer) drawingarea_ex, "button_release_event",
-                    G_CALLBACK (on_drawingarea_ex_button_release_event),
-                    NULL);
-  g_signal_connect ((gpointer) drawingarea_ex, "configure_event",
-                    G_CALLBACK (on_drawingarea_ex_configure_event),
-                    NULL);
-  g_signal_connect ((gpointer) drawingarea_ex, "expose_event",
-                    G_CALLBACK (on_drawingarea_ex_expose_event),
-                    NULL);
-  g_signal_connect ((gpointer) drawingarea_ex, "motion_notify_event",
-                    G_CALLBACK (on_drawingarea_ex_motion_notify_event),
-                    NULL);
-  g_signal_connect ((gpointer) window_ex_togglebutton, "toggled",
-		    G_CALLBACK (on_window_ex_togglebutton_toggled),
-		    NULL);
-
-  smallimage_ex = gdk_image_new (GDK_IMAGE_FASTEST, gdkvis,
-				 server_CB.client_grp[clientNum]->serverInitMsg.framebufferWidth,
-				 server_CB.client_grp[clientNum]->serverInitMsg.framebufferHeight);
-
-  return window;
-}
-
-
 gboolean
 on_window_ex_key_press_event             (GtkWidget       *widget,
                                         GdkEventKey     *event,
@@ -961,6 +899,26 @@ on_window_ex_focus_out_event             (GtkWidget       *widget,
   return TRUE;
 }
 
+gboolean
+on_aspectframe_configure_event         (GtkWidget       *widget,
+                                        GdkEventConfigure *event,
+                                        gpointer         user_data)
+{
+
+  return FALSE;
+}
+
+
+gboolean
+on_aspectframe_expose_event            (GtkWidget       *widget,
+                                        GdkEventExpose  *event,
+                                        gpointer         user_data)
+{
+
+  return FALSE;
+}
+
+
 
 void
 on_window_ex_togglebutton_toggled       (GtkToggleButton *togglebutton,
@@ -1229,6 +1187,7 @@ on_screen_expand_activate              (GtkMenuItem     *menuitem,
   clientNum = (int)user_data;
   SETCTRL(clientNum, FALSE);
   expanding_screen();
+  GtkWidget *window_ex_togglebutton = lookup_widget(GTK_WIDGET(window_ex), "window_ex_togglebutton");
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(window_ex_togglebutton), FALSE);
 }
 
@@ -1239,14 +1198,15 @@ on_screen_ctrl_activate                (GtkMenuItem     *menuitem,
 {
   clientNum = (int)user_data;
   expanding_screen();
-  if (!GETCTRL(clientNum_ex)) {
-    SETCTRL(clientNum_ex, TRUE);
+  GtkWidget *window_ex_togglebutton = lookup_widget(GTK_WIDGET(window_ex), "window_ex_togglebutton");
+  if (!GETCTRL(clientNum)) {
+    SETCTRL(clientNum, TRUE);
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(window_ex_togglebutton), TRUE);
   } else {
-    SETCTRL(clientNum_ex, FALSE);
+    SETCTRL(clientNum, FALSE);
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(window_ex_togglebutton), FALSE);
   }
-  printf("clientNum_ex=%d\n", clientNum);
+  //printf("clientNum=%d\n", clientNum);
 }
 
 
@@ -1306,6 +1266,7 @@ on_screen_send_stop_activate           (GtkMenuItem     *menuitem,
 void makeCombobox1_dialog_send(void)
 {
   int pCnt;
+  GtkWidget *dialog_send_hbox = lookup_widget(GTK_WIDGET(dialog_send), "dialog_send_hbox");
 
   dialog_send_comboboxentry1 = gtk_combo_box_entry_new_text ();
   gtk_widget_show (dialog_send_comboboxentry1);
@@ -1327,6 +1288,7 @@ void makeCombobox1_dialog_send(void)
  */
 void makeCombobox2_dialog_send(void)
 {
+  GtkWidget *dialog_send_hbox = lookup_widget(GTK_WIDGET(dialog_send), "dialog_send_hbox");
   dialog_send_comboboxentry2 = gtk_combo_box_entry_new_text ();
   gtk_widget_show (dialog_send_comboboxentry2);
   gtk_box_pack_start (GTK_BOX (dialog_send_hbox), dialog_send_comboboxentry2, TRUE, TRUE, 0);
@@ -1582,6 +1544,7 @@ on_dialog_send_comboboxentry1_changed      (GtkComboBox     *combobox,
 {
   int p, i, num;
   char *string;
+  GtkWidget *dialog_send_hbox = lookup_widget(GTK_WIDGET(dialog_send), "dialog_send_hbox");
 
   if (src_checkbutton != NULL)
     gtk_toggle_button_set_inconsistent (GTK_TOGGLE_BUTTON (src_checkbutton), FALSE);
@@ -1754,7 +1717,7 @@ send_start(void)
 	calcRateCallback ((gpointer)c_num);
 	SendAllowMsg(c_num, server_CB.client_grp[c_num]->client_ID);
       } else {
-	(*shm_asym[c_num]).flag = 2;
+	(*shm_asym[c_num]).flag = CAST_STOP;
 	int shmctlck;
 	int res = shmdt(shm_asym[c_num]);
 	shmctlck = shmctl(server_CB.client_grp[c_num]->shm_id, IPC_RMID, 0);
@@ -1816,6 +1779,9 @@ on_dialog_send_buttonApply_clicked         (GtkButton       *button,
 {
   int Cnt, pCnt, dCnt;
   int user_cnt = 0;
+  GtkWidget *dialog_send_hbox = lookup_widget(GTK_WIDGET(dialog_send), "dialog_send_hbox");
+  GtkWidget *dialog_send_viewport1 = lookup_widget(GTK_WIDGET(dialog_send), "dialog_send_viewport1");
+  GtkWidget *dialog_send_viewport2 = lookup_widget(GTK_WIDGET(dialog_send), "dialog_send_viewport2");
 
   send_start();
 
@@ -2056,3 +2022,26 @@ on_dialog_send_window_radiobutton_toggled
   fullscreen_flag = 0;
 }
 
+
+/*******************************************************************
+ *ΥǻѤؿ
+ *******************************************************************/
+
+gboolean
+on_dialog_notice_delete_event          (GtkWidget       *widget,
+                                        GdkEvent        *event,
+                                        gpointer         user_data)
+{
+  gtk_widget_destroy(dialog_notice);
+  return TRUE;
+}
+
+
+void
+on_dialog_notify_okbutton_clicked      (GtkButton       *button,
+                                        gpointer         user_data)
+{
+  gtk_widget_destroy(dialog_notice);
+}
+
+
