@@ -8,6 +8,8 @@
 #include <errno.h>
 #include <time.h>
 #include <getopt.h>
+#include <sys/types.h>
+#include <signal.h>
 
 #include "vncviewer.h"
 #include <jpeglib.h>
@@ -38,6 +40,8 @@ int lock_time = 0;
 gint lock_tag;
 int auto_flag = 0;
 int notice_flag = 1;
+char connect_server[IDLENGTH] = "\0";
+int server_flag = 0;	// init:0, -s opt set:1, sever ID match:2
 GtkWidget *drawing_window;
 GtkWidget *dialog_IDinput;
 GtkWidget *dialog_wait;
@@ -55,6 +59,12 @@ static char buf[BUF_SIZE];
 static char *bufoutptr = buf;
 static int buffered = 0;
 
+static int changePixelFormat8To16(char *src_buf, int src_size, char *new_buf);
+static int changePixelFormat8To32(char *src_buf, int src_size, char *new_buf);
+static int changePixelFormat16To8(char *src_buf, int src_size, char *new_buf);
+static int changePixelFormat16To32(char *src_buf, int src_size, char *new_buf);
+static int changePixelFormat32To8(char *src_buf, int src_size, char *new_buf);
+static int changePixelFormat32To16(char *src_buf, int src_size, char *new_buf);
 
 /* from rfbproto.c */
 static Bool HandleRRE8(int rx, int ry, int rw, int rh);
@@ -63,9 +73,15 @@ static Bool HandleRRE32(int rx, int ry, int rw, int rh);
 static Bool HandleCoRRE8(int rx, int ry, int rw, int rh);
 static Bool HandleCoRRE16(int rx, int ry, int rw, int rh);
 static Bool HandleCoRRE32(int rx, int ry, int rw, int rh);
-static Bool HandleHextile8(int rx, int ry, int rw, int rh);
-static Bool HandleHextile16(int rx, int ry, int rw, int rh);
-static Bool HandleHextile32(int rx, int ry, int rw, int rh);
+static Bool HandleHextile8To8(int rx, int ry, int rw, int rh);
+static Bool HandleHextile8To16(int rx, int ry, int rw, int rh);
+static Bool HandleHextile8To32(int rx, int ry, int rw, int rh);
+static Bool HandleHextile16To8(int rx, int ry, int rw, int rh);
+static Bool HandleHextile16To16(int rx, int ry, int rw, int rh);
+static Bool HandleHextile16To32(int rx, int ry, int rw, int rh);
+static Bool HandleHextile32To8(int rx, int ry, int rw, int rh);
+static Bool HandleHextile32To16(int rx, int ry, int rw, int rh);
+static Bool HandleHextile32To32(int rx, int ry, int rw, int rh);
 static Bool HandleZlib8(int rx, int ry, int rw, int rh);
 static Bool HandleZlib16(int rx, int ry, int rw, int rh);
 static Bool HandleZlib32(int rx, int ry, int rw, int rh);
@@ -221,23 +237,55 @@ JpegSetSrcManager(j_decompress_ptr cinfo, CARD8 *compressedData,
 #define BPP 8
 #include "mrre.c"
 #include "mcorre.c"
-#include "mhextile.c"
 #include "mzlib.c"
 #include "mtight.c"
+#define BPPC 8
+#include "mhextile.c"
+#undef BPPC
+#define BPPC 16
+#include "mhextile.c"
+#include "pixel.c"
+#undef BPPC
+#define BPPC 32
+#include "mhextile.c"
+#include "pixel.c"
+#undef BPPC
 #undef BPP
+
 #define BPP 16
 #include "mrre.c"
 #include "mcorre.c"
-#include "mhextile.c"
 #include "mzlib.c"
 #include "mtight.c"
+#define BPPC 8
+#include "mhextile.c"
+#include "pixel.c"
+#undef BPPC
+#define BPPC 16
+#include "mhextile.c"
+#undef BPPC
+#define BPPC 32
+#include "mhextile.c"
+#include "pixel.c"
+#undef BPPC
 #undef BPP
+
 #define BPP 32
 #include "mrre.c"
 #include "mcorre.c"
-#include "mhextile.c"
 #include "mzlib.c"
 #include "mtight.c"
+#define BPPC 8
+#include "mhextile.c"
+#include "pixel.c"
+#undef BPPC
+#define BPPC 16
+#include "mhextile.c"
+#include "pixel.c"
+#undef BPPC
+#define BPPC 32
+#include "mhextile.c"
+#undef BPPC
 #undef BPP
 
 #define OPTNUM 2
@@ -261,7 +309,7 @@ int client_main(int argc, char **argv)
 
   /* command line check */
   programName = argv[0];
-  while ((c = getopt_long(argc, argv, "d:u:fh", options, &option_index)) != EOF) {
+  while ((c = getopt_long(argc, argv, "d:s:u:fh", options, &option_index)) != EOF) {
 //  while ((c = getopt(argc, argv, "d:u:fnh")) != EOF) {
     switch (c) {
       case 1:
@@ -270,6 +318,10 @@ int client_main(int argc, char **argv)
       case 'd':
 	vncdisplay = optarg;
 	break;
+      case 's':
+	strcpy(connect_server, optarg);
+	server_flag = 1;
+        break;
       case 'u':
 	strcpy(client_CB.client_ID, optarg);
 	auto_flag = 1;
@@ -400,9 +452,9 @@ JoinVNC () {
   fprintf (stdout, "%s\n", commandvnc3);
 
   /* execute vncconnect */
-  client_CB.vncdsp = system(commandvnc1);
-  client_CB.vncdsp = system(commandvnc2);
-  client_CB.vncdsp = system(commandvnc3);
+  client_CB.vncdsp = (FILE*)system(commandvnc1);
+  client_CB.vncdsp = (FILE*)system(commandvnc2);
+  client_CB.vncdsp = (FILE*)system(commandvnc3);
 
   client_CB.connectstate = STAT_VNC;
 }
@@ -481,11 +533,10 @@ MultiRecieve1 ()
   numrcv = recvfrom(client_CB.mltdsp, &serveradv, sizeof(ServerAdv), 0, \
       		    (struct sockaddr *)&recvadd, (int *)&addr_len);
   memcpy(client_CB.server_ID, serveradv.server_ID, 16);
-  if (auto_flag) {
-    //client_CB.server_ip = inet_addr(SERVER_IP);
-    //client_CB.server_port = htons(SERVER_PORT);
-    client_CB.server_ip = recvadd.sin_addr.s_addr;
-    client_CB.server_port = serveradv.server_port;
+
+  if (strcmp(client_CB.server_ID, client_CB.client_ID) == 0) {
+    client_CB.server_ip = inet_addr(SERVER_IP);
+    client_CB.server_port = htons(SERVER_PORT);
   } else {
     client_CB.server_ip = recvadd.sin_addr.s_addr;
     client_CB.server_port = serveradv.server_port;
@@ -504,18 +555,26 @@ MultiRecieve1 ()
   fprintf(stderr, "serverip=%s\n",(char *)inet_ntoa(recvadd.sin_addr));
   fprintf(stderr, "serverport=%d\n\n",ntohs(serveradv.server_port));
 
-  if (auto_flag == 0) {
+  if (!strcmp(connect_server, serveradv.server_ID)) {
+    server_flag = 2;
+  }
+
+  /* no option (-u and -s unset) */
+  if ((auto_flag == 0) && (server_flag == 0)) {
     draw_string_to_mini_window("Connect?");
     open_dialog_recvAdv(serveradv.server_ID);
-  } else {
+  }
+  /* -u set and -s unset. -s set and match server_ID */
+  else if (((auto_flag == 1) && (server_flag == 0)) || (server_flag == 2)) {
     draw_string_to_mini_window("Connect");
   }
 
-  if (((answer == 'y') || (auto_flag == 1)) && JoinCTL ()) {
+  if (((answer == 'y') || ((auto_flag == 1) && (server_flag == 0)) || (server_flag == 2)) && JoinCTL ()) {
     client_CB.connectstate = STAT_ADV;  
     return True;
   } 
   
+  /* -s set and no match server_ID -> infinity loop */
   bzero(client_CB.server_ID, sizeof(client_CB.server_ID));
   client_CB.server_ip = 0;
   client_CB.server_port = 0;
@@ -611,25 +670,15 @@ GetMultiVNCServerMsg (void *g_dsp)
 	if ((client_CB.connectstate == STAT_VNC) || (client_CB.connectstate == STAT_CUT)) {
 	  if ((buf[1] == 1) && (!GETRECV_C)) {
 	    SETRECV_C(1);
+
+	    /* Stock data from MultiVNC server */
 	    client_CB.casting_ip = castadv.casting_ip;
 	    client_CB.casting_port = castadv.casting_port;
 	    memcpy (&client_CB.serverInitMsg, &castadv.serverInitMsg, sizeof(rfbServerInitMsg));
 	    memcpy(&si, &client_CB.serverInitMsg, sizeof(rfbServerInitMsg));
 	    draw_string_to_mini_window("Receive Data");
-	    if (gdkimage[0] == NULL) {
-	      gdkimage[0] = gdk_image_new(GDK_IMAGE_FASTEST, gdkvis,
-				client_CB.serverInitMsg.framebufferWidth, client_CB.serverInitMsg.framebufferHeight);
-	    }
-	    if (smallimage[0] == NULL) {
-	      smallimage[0] = gdk_image_new(GDK_IMAGE_FASTEST, gdkvis,
-				client_CB.serverInitMsg.framebufferWidth, client_CB.serverInitMsg.framebufferHeight);
-	    }
-	    drawing_window = (GtkWidget *)create_drawing_window();
-	    gtk_window_set_default_size(GTK_WINDOW(drawing_window), 600, 400);
 
-	    gtk_label_set_text(GTK_LABEL(screen_label), castadv.src_ID);
-
-	    drawingarea[0][0] = client_drawingarea;
+	    /* Create Multicast packet */
 	    printf("ip=%s, port=%d\n",(char *)inet_ntoa(client_CB.casting_ip), \
 							ntohs(client_CB.casting_port));
             client_CB.cstdsp = CreateMultiSockToRecv ((char *)inet_ntoa(client_CB.casting_ip), \
@@ -639,10 +688,20 @@ GetMultiVNCServerMsg (void *g_dsp)
 	    client_CB.g_cstdsp = gdk_input_add (client_CB.cstdsp, GDK_INPUT_READ, \
 		                                (GdkInputFunction)GetMultiVNCServerMsg, \
 						(gpointer)client_CB.cstdsp);
+	    /* Create window for drawing */
+	    drawing_window = (GtkWidget *)create_drawing_window();
+	    gtk_window_set_default_size(GTK_WINDOW(drawing_window), 600, 400);
+
+	    gtk_label_set_text(GTK_LABEL(screen_label), castadv.src_ID);
+
+	    drawingarea[0][0] = client_drawingarea;
 	    gtk_widget_show(drawing_window);
 	    if (castadv.fullscreen)
 	      gdk_window_fullscreen(gdk_window_get_toplevel(drawing_window->window));
+	    /* Initialize for drawing */
+	    memcpy(&client_CB.serverFormat, &castadv.pixelFormat, sizeof(rfbPixelFormat));
 	    DrawingInit();
+
 	  } else if ((buf[1] == 0) && (GETRECV_C)) {
 	    SETRECV_C(0);
 	    gdk_input_remove(client_CB.g_cstdsp);
@@ -674,11 +733,11 @@ GetMultiVNCServerMsg (void *g_dsp)
 	  if (buf[1]) {
 	    if (lock_window == NULL)
 	      create_lock_window();
-	    else { 
+	    else {
 	      start_lock();
 	      gtk_timeout_remove (lock_tag);
 	    }
-	    lock_tag = gtk_timeout_add((lockadv.locktime + 10) * 1000, stop_lock, NULL);
+	    lock_tag = gtk_timeout_add((int)(lockadv.locktime + 10) * 1000, (GtkFunction)stop_lock, NULL);
 	  } else {
 	    gtk_timeout_remove (lock_tag);
 	    stop_lock();
@@ -687,6 +746,20 @@ GetMultiVNCServerMsg (void *g_dsp)
 	break;
       } 
 
+      case rfbRecAdv:
+      {
+	RecAdv recadv;
+
+	printf("switch : rfbRecAdv\n");
+	if (read (client_CB.ctldsp, (char *)&recadv + 2, sizeof(RecAdv)-2) <= 0) {
+	  CloseVNCServer();
+	  fprintf(stderr, "%s : session closed\n", programName);
+	  exit (1);
+	}
+	
+	break;
+      }
+
       case rfbVncConnAdv:
         client_CB.connectstate = buf[1];
 	break;
@@ -972,18 +1045,54 @@ HandleMultiVNCServerMessage()
 
       case rfbEncodingHextile:
       {
-	switch (myFormat.bitsPerPixel) {
+	switch (client_CB.serverFormat.bitsPerPixel) {
 	case 8:
-	  if (!HandleHextile8(rect.r.x,rect.r.y,rect.r.w,rect.r.h))
-	    return False;
+	  switch (myFormat.bitsPerPixel) {
+	  case 8:
+	    if (!HandleHextile8To8(rect.r.x,rect.r.y,rect.r.w,rect.r.h))
+	      return False;
+	    break;
+	  case 16:
+	    if (!HandleHextile8To16(rect.r.x,rect.r.y,rect.r.w,rect.r.h))
+	      return False;
+	    break;
+	  case 32:
+	    if (!HandleHextile8To32(rect.r.x,rect.r.y,rect.r.w,rect.r.h))
+	      return False;
+	    break;
+	  }
 	  break;
 	case 16:
-	  if (!HandleHextile16(rect.r.x,rect.r.y,rect.r.w,rect.r.h))
-	    return False;
+	  switch (myFormat.bitsPerPixel) {
+	  case 8:
+	    if (!HandleHextile16To8(rect.r.x,rect.r.y,rect.r.w,rect.r.h))
+	      return False;
+	    break;
+	  case 16:
+	    if (!HandleHextile16To16(rect.r.x,rect.r.y,rect.r.w,rect.r.h))
+	      return False;
+	    break;
+	  case 32:
+	    if (!HandleHextile16To32(rect.r.x,rect.r.y,rect.r.w,rect.r.h))
+	      return False;
+	    break;
+	  }
 	  break;
 	case 32:
-	  if (!HandleHextile32(rect.r.x,rect.r.y,rect.r.w,rect.r.h))
-	    return False;
+	  switch (myFormat.bitsPerPixel) {
+	  case 8:
+	    if (!HandleHextile32To8(rect.r.x,rect.r.y,rect.r.w,rect.r.h))
+	      return False;
+	    break;
+	  case 16:
+	    if (!HandleHextile32To16(rect.r.x,rect.r.y,rect.r.w,rect.r.h))
+	      return False;
+	    break;
+	  case 32:
+	    if (!HandleHextile32To32(rect.r.x,rect.r.y,rect.r.w,rect.r.h))
+	      return False;
+	    break;
+	  }
 	  break;
 	}
 	break;
@@ -1049,6 +1158,7 @@ help (char *cmd)
 {
   fprintf(stderr, "\nmultivnc_client\n");
   fprintf(stderr, "  -d [num]\t\tDisplay number of vncserver\n\n");
+  fprintf(stderr, "  -s [server_name]\tInput server ID\n\n");
   fprintf(stderr, "  -u [user_name]\tInput client ID and auto connect.\n\n");
   fprintf(stderr, "  --nonotice\t\tState window is not displayed.\n\n");
 }
@@ -1079,3 +1189,18 @@ CloseVNCServer()
   }
 }
 
+int
+adjustColorBitShift(short server_bits, short client_bits)
+{
+  int i, count = 0;
+  short bits;
+
+  bits = server_bits ^ client_bits;
+  for (i = 0; i < 8; i++) {
+    count = count + (bits & 0x01);
+    bits = bits >> 1;
+  }
+
+  return count;
+}
+
